from typing import Protocol, runtime_checkable

from perika.game.text import TaskText


@runtime_checkable
class TextEngine(Protocol):
    def get_or_generate(self) -> TaskText:
        raise NotImplementedError



from __future__ import annotations

from enum import Enum, unique


@unique
class LevelComplexity(Enum):
    easy = 1
    medium = 2
    hard = 3

    @classmethod
    def list_keys(cls) -> list[str]:
        return [x.name for x in cls]

    @classmethod
    def list_value(cls) -> list[int]:
        return [x.value for x in cls]



from dataclasses import dataclass, field

import requests

from perika.game.text import TaskText


@dataclass
class FishTextRequestParams:
    format: str = "json"
    number: int = 1
    type: str = "title"


@dataclass
class FishTextRequest:
    DOMAIN: str = "https://fish-text.ru/get"
    params: FishTextRequestParams = field(default_factory=FishTextRequestParams)


class FishTextEngine:
    """https://fish-text.ru/api"""

    def __init__(self, complexity: int = 1) -> None:
        """Complexity := (title(1), sentence(2), paragraph(3))"""
        self.complexity = complexity
        if complexity not in (1, 2, 3):
            msg = "Сложность текста - это три уровня. Укажите сложность из множества (1,2,3)"
            raise AttributeError(msg)

    def _fishtext_request_url_builder(self) -> FishTextRequest:
        """ """
        complexity = {1: "title", 2: "sentence", 3: "paragraph"}

        return FishTextRequest(
            params=FishTextRequestParams(type=complexity[self.complexity]),
        )

    def _get_text(self) -> str:
        with requests.session() as session:
            prm = self._fishtext_request_url_builder()

            response = session.get(url=prm.DOMAIN, params=prm.params.__dict__)

            response.raise_for_status()
            return response.json()["text"]

    def text_status_info(self) -> str:
        return f"complexity = {self.complexity}"

    def get_or_generate(self) -> TaskText:
        return TaskText(self._get_text())



from __future__ import annotations

from typing import TYPE_CHECKING

from perika.engine.fishtext import FishTextEngine
from perika.game.choises import LevelComplexity
from perika.game.level import Level
from perika.game.task import Task

if TYPE_CHECKING:
    from perika.engine.base import TextEngine
    from perika.game.player import Player


class Game:
    def __init__(
        self,
        level_hard: LevelComplexity,
        level_size: int,
        player: Player,
        engine_name: str,
    ) -> None:
        self.level_hard = level_hard
        self.level_size = level_size
        self.player = player
        self.engine = self._resolve_engine(engine_name)

    def _resolve_engine(self, eng: str) -> TextEngine:
        if eng == "fishtext":
            return FishTextEngine(complexity=LevelComplexity[self.level_hard].value)  # type: ignore
        msg = "Engine not implemented"
        raise NotImplementedError(msg)

    def generate_level(self) -> Level:
        result = [Task(self.engine.get_or_generate()) for _ in range(self.level_size)]
        return Level(result)



import difflib
from dataclasses import dataclass
from typing import Optional

from perika.game.text import PlayerAnswer, TaskText


@dataclass
class TaskResult:
    equal: bool
    diff: Optional[str]

    def __repr__(self) -> str:
        return f"TaskResult \n equal >>> {self.equal}\n diff  >>> \n {self.diff}"


class Task:
    def __init__(self, text: TaskText) -> None:
        self.text = text

    def __call__(self) -> str:
        return self.text()

    def compare(self, player_input: PlayerAnswer) -> TaskResult:
        eq = self.text() == player_input()
        diff = "Success"
        if not eq:
            differ = difflib.Differ()
            raw_diff = list(differ.compare([self.text()], [player_input()]))
            diff = "\n".join(raw_diff)
        return TaskResult(equal=eq, diff=diff)



from typing import NoReturn


class Player:
    def __init__(self, name: str) -> None:
        self.name = name

    def sync(self) -> NoReturn:
        raise NotImplementedError

    def metadata(self) -> NoReturn:
        raise NotImplementedError



from typing import List

from typing_extensions import Self

from .task import Task


class Level:
    def __init__(self, tasks: List[Task]) -> None:
        self.tasks = tasks
        self.index = 0
        self.max_level_size = 10

    def __iter__(self) -> Self:
        return self

    def __next__(self) -> Task:
        if self.index < len(self.tasks) and self.index <= self.max_level_size:
            item = self.tasks[self.index]
            self.index += 1
            return item
        raise StopIteration



import time
from dataclasses import dataclass

from typing_extensions import Self

from perika.game.task import TaskResult


@dataclass
class TaskResultWithTime:
    result: TaskResult
    time: float

    def __repr__(self) -> str:
        return str(self.result) + f"\n User prompt time: {self.time:.2f} sec"


class LevelTimer:
    def __init__(self, result_pattern: str = "{:.2f}") -> None:
        self.result_pattern: str = result_pattern

    def __enter__(self) -> Self:
        self.start_time = time.time()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):  # noqa: ANN204, ANN001
        self.end_time: float = time.time() - self.start_time

    def result(self, result: TaskResult) -> TaskResultWithTime:
        return TaskResultWithTime(result=result, time=self.end_time)



import functools


@functools.lru_cache
def _clean(text: str) -> str:
    return text.strip()


class TaskText:
    def __init__(self, text: str) -> None:
        self.text = text

    def __call__(self) -> str:
        return self.text


class PlayerAnswer:
    def __init__(self, player_text: str) -> None:
        self.player_text = player_text

    def __call__(self) -> str:
        return self.clean()

    def clean(self) -> str:
        self.player_text = _clean(self.player_text)
        return self.player_text



import typer
from rich import print
from rich.panel import Panel
from rich.progress import Progress, SpinnerColumn, TextColumn
from rich.prompt import IntPrompt, Prompt

from perika.clock import LevelTimer
from perika.game.choises import LevelComplexity
from perika.game.player import Player
from perika.game.rule import Game
from perika.game.text import PlayerAnswer


def start_game() -> None:
    name = Prompt.ask(
        prompt="Enter your name :waving_hand:",
        default="Guest",
        show_default=False,
    )

    print(f"Hello [green]{name}![/green]")

    lvl_hard = Prompt.ask(
        prompt="Enter level hard :flexed_biceps:",
        choices=LevelComplexity.list_keys(),
        default=LevelComplexity.easy.name,
        show_choices=True,
    )

    lvl_size = IntPrompt.ask("Enter level size (int, max: 10) :sunglasses:", default=1, show_default=False)

    player = Player(name)

    engine_name = Prompt.ask(
        "Set text engine :brain:",
        default="fishtext",
        choices=["fishtext", "gigachat"],
        show_choices=True,
    )

    game_rule = Game(lvl_hard, lvl_size, player, engine_name)  # type: ignore

    with Progress(
        SpinnerColumn(),
        TextColumn("[progress.description]{task.description}"),
        transient=True,
    ) as progress:
        progress.add_task(description="Generate level...", total=None)
        game_level = game_rule.generate_level()

    print(
        Panel(
            f"Game information! :flag_in_hole: \n\n "
            f"Player name: [red]{name}[/red] \n "
            f"Level hard: [yellow]{lvl_hard.capitalize()} [/yellow]\n "
            f"Level size: [green]{lvl_size}[/green]\n "
            f"Text generating engine: [blue]{engine_name.capitalize()}[/blue]",
            title="Game level info",
        ),
    )

    start_game = typer.confirm("Start the game?", default=True, show_default=True)

    if not start_game:
        msg = "Bye-bye .. game cancelled :("
        raise typer.Abort(msg)

    cnt = 1
    for task in game_level:
        print(Panel.fit(task(), title=f"Round {cnt}"))
        with LevelTimer() as timer:
            answer = PlayerAnswer(Prompt.ask("[bold red] your prompt -> "))

        print(timer.result(task.compare(answer)))
        cnt += 1
